# -*- coding: utf-8 -*-

# note: magics in here just to check they do not directly cause
# errors.

nb_basic = u'''
{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "%(the_source)s"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "%%env"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "%%%%writefile sigh\\n",
    "1"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "u\\"中国\\""
   ]
  }
 ],
 "metadata": {
  "language_info": {
   "name": "python",
   "pygments_lexer": "ipython3"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
'''

WARNINGS_ARE_ERRORS = '-W error'
VERBOSE = '-v'
# Ignore deprecation warnings until the many deprecation warnings stop
# coming from ipython and traitlets as used by nbconvert
IGNORE_DEPRECATION_WARNINGS = '-W ignore::DeprecationWarning'

_all_args = [
    VERBOSE,
    WARNINGS_ARE_ERRORS,
    IGNORE_DEPRECATION_WARNINGS
]


lint_args = ['--nbsmoke-lint'] + _all_args

run_args = ['--nbsmoke-run'] + _all_args



# -*- coding: utf-8 -*-

import os
import io
import sys
import shutil

from . import nb_basic, run_args, WARNINGS_ARE_ERRORS

# tests are run in subprocess because otherwise some state seems to be left
# around somewhere in jupyter (https://github.com/pyviz-dev/nbsmoke/issues/45)

# maybe this test is overkill now we check for certain output in the run tests?
def test_definitely_ran_paranoid(testdir):
    assert not os.path.exists('sigh')
    testdir.makefile('.ipynb', testing123=nb_basic%{'the_source':"open('x','w').write('y')"})
    # Suppress reporting test as failed when a warning is emitted,
    # due to ResourceWarning raised by pyzmq. This should be removed later.
    args = run_args.copy(); args.remove(WARNINGS_ARE_ERRORS)
    result = testdir.runpytest_subprocess(*args)
    assert result.ret==0
    with open('x','r') as f:
        assert f.read() == 'y'
    assert os.path.isfile('sigh')

def test_run_good(testdir):
    testdir.makefile('.ipynb', testing123=nb_basic%{'the_source':"1/1"})
    # Suppress reporting test as failed when a warning is emitted,
    # due to ResourceWarning raised by pyzmq. This should be removed later.
    args = run_args.copy(); args.remove(WARNINGS_ARE_ERRORS)
    result = testdir.runpytest_subprocess(*args)
    assert result.ret == 0
    result.stdout.re_match_lines_random(
        [".*collected 1 item$",
         ".*testing123.ipynb.*PASSED.*"])

def test_run_bad(testdir):
    testdir.makefile('.ipynb', testing123=nb_basic%{'the_source':"1/0"})
    result = testdir.runpytest_subprocess(*run_args)
    assert result.ret == 1
    result.stdout.re_match_lines_random([".*ZeroDivisionError.*"])

def test_run_good_html(testdir):
    outhtml = os.path.join(testdir.tmpdir.strpath,'testing123.ipynb.html')
    assert not os.path.exists(outhtml)

    testdir.makefile('.ipynb', testing123=nb_basic%{'the_source':"42"})

    args = run_args + ['--store-html=%s'%testdir.tmpdir.strpath]
    # nbconvert.HTMLExporter.from_notebook_node seems to be raising
    # a ResourceWarning that is caught by pytest and causes the test
    # to fail (the test suite fails if a warning is emitted). pytest
    # catches this kind of warning (unraisable) from Python 3.8.
    if sys.version_info >= (3, 8):
        args += ['-p', 'no:unraisableexception']
    result = testdir.runpytest_subprocess(*args)
    assert result.ret == 0

    # test that html has happened
    targets = [
        "<pre>42</pre>",
                                               # note: this is really what happens in a python2 notebook
        "<pre>&#39;中国&#39;</pre>" if sys.version_info[0]>=3 else r"<pre>u&#39;\u4e2d\u56fd&#39;</pre>"]
    answer = [None,None]

    with io.open(outhtml,encoding='utf8') as f:
        for line in f:
            for i,target in enumerate(targets):
                if target in line:
                    answer[i] = 42
    assert answer == [42,42]


def test_skip_run(testdir):
    testdir.makeini(r"""
        [pytest]
        nbsmoke_skip_run = ^.*skipme\.ipynb$
                           ^.*skipmetoo.*$
    """)
    testdir.makefile('.ipynb', skipme=nb_basic%{'the_source':"1/0"})
    testdir.makefile('.ipynb', alsoskipme=nb_basic%{'the_source':"1/0"})
    testdir.makefile('.ipynb', skipmetoo=nb_basic%{'the_source':"1/0"})
    testdir.makefile('.ipynb', skipmenot=nb_basic%{'the_source':"1/1"})
    # Suppress reporting test as failed when a warning is emitted,
    # due to ResourceWarning raised by pyzmq. This should be removed later.
    args = run_args.copy(); args.remove(WARNINGS_ARE_ERRORS)
    result = testdir.runpytest_subprocess(*args)
    assert result.ret == 0
    result.stdout.re_match_lines_random(
        [".*collected 4 items$",
         ".*alsoskipme.ipynb.*SKIPPED",
         ".*skipme.ipynb.*SKIPPED",
         ".*skipmenot.ipynb.*PASSED",
         ".*skipmetoo.ipynb.*SKIPPED"])

def test_cwd_like_jupyter_notebook(testdir):
    p = testdir.tmpdir.mkdir("sub").join("hello.txt")
    p.write("content")
    testdir.makefile('.ipynb', testing123=nb_basic%{'the_source':"import os; assert os.path.isfile('hello.txt')"})
    shutil.move('testing123.ipynb', 'sub')
    # Suppress reporting test as failed when a warning is emitted,
    # due to ResourceWarning raised by pyzmq. This should be removed later.
    args = run_args.copy(); args.remove(WARNINGS_ARE_ERRORS)
    result = testdir.runpytest_subprocess(*args)
    assert result.ret == 0



from IPython.utils._process_common import arg_split
import argparse

IGNORED_LINE_MAGICS = [
#    'timer',  ???
    'bookmark',
    'colors',
    'conda',
    'config',
    'debug',
    'dhist',
    'doctest_mode',
    'edit',
    'env',
    'gui',
    'history',
    'killbgscripts',
    'load',
    'load_ext',
    'loadpy',
    'logon',
    'logoff',
    'logstart',
    'logstate',
    'logstop',
    'lsmagic',
    'magic',
    'matplotlib',
    'notebook',
    'page',
    'pastebin',
    'pdb',
    'pdef',
    'pdoc',
    'pfile',
    'pinfo',
    'pinfo2',
    'pip',
    'pprint',
    'precision',
    'psearch',
    'psource',
    'pycat',
    'quickref',
    'recall',
    'reload_ext',
    'rerun',
    'run',
    'save',
    'set_env',
    'unalias',
    'unload_ext',
    'who',
    'who_ls',
    'whos',
]

IGNORED_CELL_MAGICS = [
    ## other languages / don't interact with python
    'html',
    'javascript',
    'js',
    'latex',
    'markdown',
    'svg',
    ## misc / don't interact with python
    'writefile',
    'matplotlib'
]


cell_magic_handlers = {}
line_magic_handlers = {}

_capture_arg_parser = argparse.ArgumentParser()
_capture_arg_parser.add_argument('output', nargs='?', default=None)
def capture_handler(magic):
    string_of_magic_args = magic.python
    x,_ = _capture_arg_parser.parse_known_args(arg_split(string_of_magic_args))
    if x.output:
        return '%s = "dummy captured output"'%x.output
    else:
        return 'pass # was capture cell magic with no output arg'

cell_magic_handlers['capture'] = capture_handler

_script_arg_parser = argparse.ArgumentParser()
_script_arg_parser.add_argument("--out")

def script_handler(magic):
    string_of_magic_args = magic.python
    script_name, _, script_arg_s = string_of_magic_args.partition(' ')
    return _script_handler(magic, script_name, script_arg_s)

def specific_script_handler(magic):
    script_name = magic.name
    script_arg_s = magic.python
    return _script_handler(magic, script_name, script_arg_s)
    
def _script_handler(magic, script_name, script_arg_s):
    x,_ = _script_arg_parser.parse_known_args(arg_split(script_arg_s))
    # this is sneaky
    magic.additional_lines = ["pass # script content omitted"]
    if x.out:
        return '%s = "dummy output from %s script run in subprocess"'%(x.out, script_name)
    else:
        return 'pass # was %s script cell magic with no --out'%script_name

_SCRIPT_MAGICS = [
    'bash',
    'perl',
    'pypy',
    'python',
    'python2',
    'python3',
    'ruby',
    'sh'
]

cell_magic_handlers['script'] = script_handler
for _magic in _SCRIPT_MAGICS:
    cell_magic_handlers[_magic] = specific_script_handler



# -*- coding: utf-8 -*-

from . import nb_basic

def test_help_message(testdir):
    result = testdir.runpytest(
        '--help',
    )
    result.stdout.fnmatch_lines([
        'nbsmoke:',
        '*--nbsmoke-run*',
        '*--nbsmoke-lint*',
        '*--nbsmoke-lint-onlywarn*',
        '*--nbsmoke-verify*',
        '*--store-html*'
    ])


def test_nbsmoke_cell_timeout_ini_setting(testdir):
    testdir.makeini("""
        [pytest]
        nbsmoke_cell_timeout = 123
    """)

    testdir.makepyfile("""
        import pytest

        @pytest.fixture
        def nbsmoke_cell_timeout(request):
            return request.config.getini('nbsmoke_cell_timeout')

        def test_nbsmoke_cell_timeout(nbsmoke_cell_timeout):
            assert int(nbsmoke_cell_timeout) == 123
    """)

    result = testdir.runpytest('-v')

    result.stdout.fnmatch_lines([
        '*::test_nbsmoke_cell_timeout PASSED*',
    ])

    assert result.ret == 0

    
def test_it_is_nbfile(testdir):
    testdir.makeini(r"""
        [pytest]
        it_is_nb_file = ^.*\.something$
    """)
    testdir.makefile('.ipynb', testing123=nb_basic%{'the_source':"1/0"})
    result = testdir.runpytest('--nbsmoke-run','-v')
    assert result.ret == 5
    result.stdout.fnmatch_lines(['*collected 0 items*'])
